// NoMoSmooth
// Avisynth filter for motion-adaptive spatio-temporal smoothing
//
// By Ross Thomas <ross@grinfinity.com>
//
// There is no copyright on this code, and there are no conditions
// on its distribution or use. Do with it what you will.

#define MEMORY_ERROR "NoMoSmooth: Could not allocate memory"

#define WIN32_LEAN_AND_MEAN
#include "windows.h"
#include "math.h"
#include "assert.h"
#include "avisynth.h"

#include "YUY2.h"

AVSValue __cdecl Create_NoMoSmooth(AVSValue args, void* user_data,
								   IScriptEnvironment* env)
{
	enum ARGS { CLIP, MOTION_THRESHOLD, TEMPORAL_RADIUS, TEMPORAL_THRESHOLD,
		SPATIAL_RADIUS, SPATIAL_THRESHOLD, SHOW };
	
	PClip clip = args[0].AsClip();
	const VideoInfo& vi = clip->GetVideoInfo();

	if(!vi.IsYUY2())
		env->ThrowError("NoMoSmooth: Clip must be in YUY2 format");

	if(vi.width % 2)
		env->ThrowError("NoMoSmooth: Width of clip must be even");

	int motion_threshold = args[MOTION_THRESHOLD].AsInt(40),
		temporal_radius = args[TEMPORAL_RADIUS].AsInt(1),
		temporal_threshold = args[TEMPORAL_THRESHOLD].AsInt(6),
		spatial_radius = args[SPATIAL_RADIUS].AsInt(1),
		spatial_threshold = args[SPATIAL_THRESHOLD].AsInt(3);
	bool show = args[SHOW].AsBool(false);

	return new NoMoSmooth_YUY2(args[0].AsClip(), motion_threshold,
		temporal_radius, temporal_threshold, spatial_radius,
		spatial_threshold, show, env);
}

extern "C" __declspec(dllexport) const char* __stdcall AvisynthPluginInit2(IScriptEnvironment* env)
{
	env->AddFunction("NoMoSmooth", "c[motion_threshold]i[temporal_radius]i"
		"[temporal_threshold]i[spatial_radius]i[spatial_threshold]i[show]b",
		Create_NoMoSmooth, 0);
	return "NoMoSmooth: Motion-adaptive spatio-temporal smoother";
}
